@@ -31,6 +31,7 @@
 #include <glib.h>
 #include <gdbus/gdbus.h>
 #include <gobex/gobex-apparam.h>
+#include <bluetooth/sdp.h>
 
 #include "dbus.h"
 #include "log.h"
@@ -95,6 +96,8 @@ struct map_data {
 	struct obc_session *session;
 	DBusMessage *msg;
 	GHashTable *messages;
+	int16_t mas_instance_id;
+	uint8_t supported_message_types;
 };
 
 #define MAP_MSG_FLAG_PRIORITY	0x01
@@ -181,7 +184,7 @@ static void folder_element(GMarkupParseContext *ctxt, const char *element,
 {
 	DBusMessageIter dict, *iter = user_data;
 	const char *key;
-	gint i;
+	int i;
 
 	if (strcasecmp("folder", element) != 0)
 		return;
@@ -973,7 +976,7 @@ static void msg_element(GMarkupParseContext *ctxt, const char *element,
 	DBusMessageIter entry, dict, *iter = parser->iter;
 	struct map_msg *msg;
 	const char *key;
-	gint i;
+	int i;
 
 	if (strcasecmp("msg", element) != 0)
 		return;
@@ -1408,10 +1411,10 @@ static DBusMessage *map_list_messages(DBusConnection *connection,
 	return get_message_listing(map, message, folder, apparam);
 }
 
-static char **get_filter_strs(uint64_t filter, gint *size)
+static char **get_filter_strs(uint64_t filter, int *size)
 {
 	char **list, **item;
-	gint i;
+	int i;
 
 	list = g_malloc0(sizeof(char **) * (FILTER_BIT_MAX + 2));
 
@@ -1430,7 +1433,7 @@ static DBusMessage *map_list_filter_fields(DBusConnection *connection,
 					DBusMessage *message, void *user_data)
 {
 	char **filters = NULL;
-	gint size;
+	int size;
 	DBusMessage *reply;
 
 	filters = get_filter_strs(FILTER_ALL, &size);
@@ -1729,6 +1732,28 @@ static void map_free(void *data)
 	g_free(map);
 }
 
+static void parse_service_record(struct map_data *map)
+{
+	const void *data;
+
+	/* MAS instance id */
+	map->mas_instance_id = -1;
+	data = obc_session_get_attribute(map->session,
+						SDP_ATTR_MAS_INSTANCE_ID);
+	if (data != NULL)
+		map->mas_instance_id = *(uint8_t *)data;
+	else
+		DBG("Failed to read MAS instance id");
+
+	/* Supported Message Types */
+	data = obc_session_get_attribute(map->session,
+					SDP_ATTR_SUPPORTED_MESSAGE_TYPES);
+	if (data != NULL)
+		map->supported_message_types = *(uint8_t *)data;
+	else
+		DBG("Failed to read supported message types");
+}
+
 static int map_probe(struct obc_session *session)
 {
 	struct map_data *map;
@@ -1736,8 +1761,6 @@ static int map_probe(struct obc_session *session)
 
 	path = obc_session_get_path(session);
 
-	DBG("%s", path);
-
 	map = g_try_new0(struct map_data, 1);
 	if (!map)
 		return -ENOMEM;
@@ -1746,6 +1769,10 @@ static int map_probe(struct obc_session *session)
 	map->messages = g_hash_table_new_full(g_str_hash, g_str_equal, NULL,
 								map_msg_remove);
 
+	parse_service_record(map);
+
+	DBG("%s, instance id %d", path, map->mas_instance_id);
+
 	set_notification_registration(map, true);
 
 	if (!g_dbus_register_interface(conn, path, MAP_INTERFACE, map_methods,
